@@ -279,7 +279,11 @@ void SCR_DrawCenterString (void)
 		SCR_AddDirtyPoint (x, y);
 		for (j=0 ; j<l ; j++, x+=8)
 		{
+#ifdef QMAX
+			re.DrawChar (x, y, start[j], 256);
+#else	
 			re.DrawChar (x, y, start[j]);	
+#endif
 			if (!remaining--)
 				return;
 		}
@@ -851,7 +855,11 @@ void DrawHUDString (char *string, int x, int y, int centerwidth, int xor)
 			x = margin;
 		for (i=0 ; i<width ; i++)
 		{
+#ifdef QMAX
+			re.DrawChar (x, y, line[i]^xor, 256);
+#else
 			re.DrawChar (x, y, line[i]^xor);
+#endif
 			x += 8;
 		}
 		if (*string)
